package cz.fidentis.web.async;

import cz.fidentis.analyst.Logger;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Patrik Tomov
 * @since  07.04.2024
 */
public class WebSocketProgressHandler extends TextWebSocketHandler {
    private final Map<String, WebSocketSession> sessions = new ConcurrentHashMap<>();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        sessions.put(session.getId(), session);
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) {
        sessions.remove(session.getId());
    }

    public void updateProgress(int progress) {
        TextMessage message = new TextMessage(String.valueOf(progress));

        sessions.values().forEach(session -> {
            try {
                session.sendMessage(message);
            } catch (IOException e) {
                Logger.print(e.toString());
            }
        });
    }
}
